@@ -3,15 +3,15 @@ use Mojo::Base 'Mojolicious::Plugin';
 use Mojolicious::Plugin::Vparam::Common qw(:all);
 
 use version;
-use List::MoreUtils qw(firstval natatime mesh);
+use List::MoreUtils qw(firstval);
 
-our $VERSION    = '2.11.1';
+our $VERSION    = '2.01';
 
 # Regext for shortcut parser
 our $SHORTCUT_REGEXP = qr{
     ^
     (
-        [!?@~%]                                         # symbols shortcut
+        [!?@~]                                          # symbols shortcut
         |
         (?:array|maybe|optional|required?|skipundef)\[  # text[] shortcut start
     )
@@ -38,23 +38,18 @@ sub register {
 
     $conf->{phone_country}  //= '';
     $conf->{phone_region}   //= '';
-    $conf->{phone_fix}      //= '';
 
     $conf->{date}           = '%F'          unless exists $conf->{date};
     $conf->{time}           = '%T'          unless exists $conf->{time};
     $conf->{datetime}       = '%F %T %z'    unless exists $conf->{datetime};
 
-    $conf->{blessed}        //= 1;
     $conf->{optional}       //= 0;
     $conf->{skipundef}      //= 0;
-    $conf->{multiline}      //= 0;
 
     $conf->{address_secret} //= '';
 
     $conf->{password_min}   //= 8;
 
-    $conf->{hash_delimiter} //= '=>';
-
     # Enable Mojolicious::Validator::Validation integration if available
     $conf->{mojo_validator} //=
         version->new($Mojolicious::VERSION) < version->new(4.42) ? 0 : 1;
@@ -172,14 +167,6 @@ sub register {
             ? delete $params{-skipundef}
             : $conf->{skipundef}
         ;
-        my $multiline = exists $params{-multiline}
-            ? delete $params{-multiline}
-            : $conf->{multiline}
-        ;
-        my $blessed = exists $params{-blessed}
-            ? delete $params{-blessed}
-            : $conf->{blessed}
-        ;
 
         # Internal variables
         my $vars = $self->stash->{'vparam-vars'} //= {};
@@ -213,15 +200,13 @@ sub register {
                 }
             }
 
-            # Set defaults
+            # Set default optional
             $attr{optional}     //= $optional;
+            # Set default skipundef
             $attr{skipundef}    //= $skipundef;
-            $attr{multiline}    //= $multiline;
-            $attr{blessed}      //= $blessed;
 
             # Apply type
-            my $type = $attr{type} // $attr{isa};
-            if( defined $type ) {
+            if( defined( my $type = $attr{type} ) ) {
                 # Parse shortcut
                 while( my ($mod, $inner) = $type =~ $SHORTCUT_REGEXP ) {
                     last unless $inner;
@@ -233,8 +218,6 @@ sub register {
                         $attr{optional} = 0;
                     } elsif( $mod eq '@' || $mod =~ m{^array\[}i) {
                         $attr{array}    = 1;
-                    } elsif( $mod eq '%' ) {
-                        $attr{hash}     //= $conf->{hash_delimiter};
                     } elsif(                $mod =~ m{^skipundef\[}i) {
                         $attr{skipundef}= 1;
                     } elsif( $mod eq '~' ) {
@@ -278,10 +261,7 @@ sub register {
                 if( $vars->{json} ) {
                     $vars->{pointer} //=
                         Mojo::JSON::Pointer->new( $vars->{json} );
-                    if( $vars->{pointer}->contains( $attr{jpath} ) ) {
-                        my $value = $vars->{pointer}->get( $attr{jpath} );
-                        @input = 'ARRAY' eq ref $value ? @$value : $value;
-                    }
+                    @input = $vars->{pointer}->get( $attr{jpath} );
                 }
             } elsif( $attr{cpath} ) {
                 # CSS
@@ -305,73 +285,26 @@ sub register {
                 @input = params($self, $name);
             }
 
-            # Set undefined value if parameter not set
-            # if array or hash then keep it empty
-            @input = (undef)
-                if
-                    not @input
-                and not $attr{array}
-                and not $attr{hash}
-            ;
+            # Set undefined value if paremeter not set
+            # if array then keep it empty
+            @input = (undef) if not @input and not $attr{array};
 
             # Set array if values more that one
-            $attr{array} = 1 if @input > 1 and not $attr{hash};
-
-            if( $attr{multiline} ) {
-                if( $attr{array} ) {
-                    die 'Array of arrays not supported';
-                } else {
-                    my $regexp = 'Regexp' eq ref $attr{multiline}
-                         ? $attr{multiline}
-                         : qr{\r?\n}
-                    ;
-                    # Apply multiline
-                    @input =
-                        grep { $_ =~ m{\S} }
-                        map  { split $regexp, $_ } @input;
-                }
-
-                # Multiline force array
-                $attr{array} = 1;
-            }
-
-            # Normalize hash key delimiter
-            $attr{hash} = $conf->{hash_delimiter}
-                if $attr{hash} && $attr{hash} eq '1';
+            $attr{array} = 1 if @input > 1;
 
             # Process on all input values
-            my @keys;
             my @output;
             for my $index ( 0 .. $#input ) {
                 my $in = my $out = $input[$index];
 
                 $out = $in;
 
-                my $key;
-                if( $attr{hash} ) {
-                    my @splitted = split $attr{hash}, $out, 2;
-                    unless( @splitted == 2 ) {
-                        $self->verror(
-                            $name,
-                            %attr,
-                            index   => $index,
-                            in      => $in,
-                            out     => $out,
-                            message => 'Not a hash',
-                        );
-                        next;
-                    }
-
-                    $key = $splitted[0];
-                    $out = $splitted[1];
-                }
-
                 # Apply pre filter
-                $out = $attr{pre}->( $self, $out, \%attr )   if $attr{pre};
+                $out = $attr{pre}->( $self, $out )    if $attr{pre};
 
                 # Apply validator
                 if( $attr{valid} ) {
-                    if( my $error = $attr{valid}->($self, $out, \%attr) ) {
+                    if( my $error = $attr{valid}->($self, $out)  ) {
                         # Set default value if error
                         $out = $attr{default};
 
@@ -398,12 +331,10 @@ sub register {
                 # Hack for bool values:
                 # HTML forms do not transmit if checkbox off
                 $out = $attr{default}
-                    if      $type
-                        and $type =~ m{^(?:bool|true|checkbox)$}
-                        and not defined $in;
+                    if $attr{type} && $attr{type} eq 'bool' and not defined $in;
 
                 # Apply post filter
-                $out = $attr{post}->( $self, $out, \%attr )  if $attr{post};
+                $out = $attr{post}->( $self, $out )   if $attr{post};
 
                 # Apply other filters
                 for my $key ( keys %attr ) {
@@ -438,39 +369,22 @@ sub register {
                 }
 
                 # Add output
-                if( defined($out) or not $attr{skipundef} ) {
-                    push @output,   $out;
-                    push @keys,     $key if $attr{hash} and defined $key;
-                }
+                push @output, $out
+                    unless $attr{skipundef} and not defined($out);
             }
 
-            # Error for required empty array
+            # Error for required empty arrays
             $self->verror( $name, %attr, message => 'Empty array' )
                 if $attr{array} and not $attr{optional} and not @input;
-            # Error for required empty hash
-            $self->verror( $name, %attr, message => 'Empty hash' )
-                if $attr{hash} and not $attr{optional} and not @input;
 
-            # Rename for model
-            my $as = $attr{as} // $name;
-
-            if( $attr{hash} ) {
-                $result{ $as } = { mesh @keys, @output };
-            } elsif( $attr{array} ) {
-                if( defined $attr{multijoin} ) {
-                    $result{ $as } = @output
-                        ? join $attr{multijoin}, grep {defined} @output
-                        : undef
-                    ;
-                } else {
-                    $result{ $as } = @output;
-                }
+            if( $attr{array} ) {
+                $result{ $name } = \@output;
             } else {
-                $result{ $as } = $output[0]
+                $result{ $name } = $output[0]
                     unless $attr{skipundef} and not defined($output[0]);
             }
             # Mojolicious::Validator::Validation
-            $self->validation->output->{$name} = $result{ $as }
+            $self->validation->output->{$name} = $result{ $name }
                 if $conf->{mojo_validator};
         }
 
@@ -501,7 +415,7 @@ sub register {
             $result = $self->vparams( $name => { type   => $def, %attr } );
         }
 
-        return $result->{ $attr{as} // $name };
+        return $result->{$name};
     });
 
     # Load extensions: types, filters etc.
@@ -550,7 +464,7 @@ Filters complementary types
 
 =item *
 
-Support arrays and hashes of values
+Support arrays of values
 
 =item *
 
@@ -713,17 +627,13 @@ If you set undef then parameter is not apply.
 Get parameter error string. Return 0 if no error.
 
     # Get error
-    print $self->verror('myparam');
-
-    # Get error for first element in array
-    print $self->verror('myparam' => 0);
-
+    print $self->verror('myparam') || 'Ok';
     # Set error
-    $self->verror('myparam', message => 'Error message')
+    $self->verror('myparam', {message => 'Error message'})
 
 =head2 verrors
 
-Return errors count in scalar context. In list context return errors hash.
+Return erorrs count in scalar context. In list context return erorrs hash.
 
     # List context get hash
     my %errors = $self->verrors;
@@ -843,7 +753,6 @@ Just set this parameters as HashRef with new types definition.
 You can simple add you own filters.
 Just set this parameters as HashRef with new filters definition.
 
-
 =item vsort_page
 
 Parameter name for current page number in I<vsort>. Default: page.
@@ -876,10 +785,6 @@ Phone country. Default: empty.
 
 Phone region. Default: empty.
 
-=item phone_fix
-
-Name of algorithm to fix phone, typicallty country code. Default: empty.
-
 =item date
 
 Date format for strftime. Default: %F.
@@ -895,11 +800,6 @@ if no format specified, return L<DateTime> object.
 Datetime format for strftime. Default: '%F %T %z'.
 if no format specified, return L<DateTime> object.
 
-=item blessed
-
-By default return objects used for parse or validation:
-L<Mojo::URL>, L<DateTime>, etc.
-
 =item optional
 
 By default all parameters are required. You can change this by set this
@@ -914,11 +814,6 @@ MD5 ensures that the coordinates belong to address.
 
 Minimum password length. Default: 8.
 
-=item hash_delimiter
-
-Delimiter to split input parameter on two parts: key and value.
-Default: => - like a perl hash.
-
 =item mojo_validator
 
 Enable L<Mojolicious::Validator::Validation> integration.
@@ -1048,10 +943,6 @@ I<TRUE> can be 1, yes, true, ok
 
 I<FALSE> can be 0, no, false, fail
 
-=item
-
-Empty string is I<FALSE>
-
 =back
 
 Other values get error.
@@ -1178,23 +1069,11 @@ Parameter type. If set then some filters will be apply. See L</TYPES>.
     $self->vparam(myparam => {type => 'datetime'});
     $self->vparams(
         myparam1 => {type => 'datetime'},
-        myparam2 => {isa  => 'datetime'},
+        myparam2 => {type => 'datetime'},
     );
 
 After the application of the type used filters.
 
-You can use B<isa> alias instead of B<type>.
-
-=head2 as
-
-Rename output key for I<vparams> to simple use in models.
-
-    # Return {login => '...', password => '...'}
-    $self->vparams(
-        myparam1 => {type => 'str', as => 'login'},
-        myparam2 => {type => 'str', as => 'password'},
-    );
-
 =head2 array
 
 Force value will array. Default: false.
@@ -1212,27 +1091,6 @@ You can force values will arrays by B<@> prefix or case insensive B<array[...]>.
     # Example: http://mysite.us?array4=123&array4=456...
     $param4 = $self->vparam(array4 => 'int');
 
-=head2 hash
-
-Get value as hash. Default: false.
-You can get values as hash by B<%> prefix.
-
-To make hash vparam split value by predefined delimiter
-(see I<hash_delimiter> configuration parameter). It`s important to make
-value format.
-
-    # Get param1 as hash. Parameter value need to be like 'a=>1'.
-    $param1 = $self->vparam(myparam1 => '%int');
-
-    # Same, but custom delimiter "::"
-    $param2 = $self->vparam(myparam2 => 'int', hash => '::');
-
-    # Multiple parameters supported.
-    # You can send many myparam3 with different values list like:
-    # ?myparam3=a_1&myparam3=b_2&myparam3=c_3
-    # and get perl hash like $param3 = {a => 1, b => 3, c => 3}
-    $param3 = $self->vparam(myparam3 => '%int', hash => '_');
-
 =head2 optional
 
 By default all parameters are required. You can change this for parameter by
@@ -1316,41 +1174,7 @@ if it`s undefined by set I<skipundef>.
     # Shortcut syntax: skipundef and optional is on
     $param2 = $self->vparam(param2 => '~int');
 
-Arrays always return as arrayref. But undefined values will be skipped:
-
-    # This vparam return [1,2,3] for ?param3=1&param3=&param3=2&param3=3
-    $param2 = $self->vparam(param3 => '~int');
-
-=head2 multiline
-
-You can simple split I<textarea> to values:
-
-    # This vparam return [1,2,3] for input "1\n2\n3\n"
-    $param1 = $self->vparam(param1 => 'int', multiline => 1);
-
-    # Or by custom regexp
-    # This vparam return [1,2,3] for input "1,2,3"
-    $param1 = $self->vparam(param1 => 'int', multiline => qr{\s*,\s*});
-
-Empty lines ignored.
-
-=head2 multijoin
-
-Any array values can be joined in string:
-
-    # This vparam return "1,2,3" for input ?param1=1&param1=2&param1=3
-    $param1 = $self->vparam(param1 => 'int', multijoin => ',');
-
-    # This vparam return "1,2,3" for input "1\n2\n3\n"
-    $param2 = $self->vparam(param2 => 'int', multiline => 1, multijoin => ',');
-
-=head2 blessed
-
-Keep and return blessed object for parsed parameters if available.
-Vparam always return scalars if disabled.
-
-Note: if defined I<date>, I<time>, I<datetime> then always return
-formatted scalar.
+Arrays always return as arrayref. But undefined values will be skipped.
 
 =head2 jpath
 
@@ -1366,8 +1190,6 @@ from L<Mojo::JSON::Pointer> to get and validate parameters.
         lat     => { type => 'lat', jpath => '/point/lat' },
     );
 
-Note: we don`t support multikey in json. Use hash or die.
-
 =head2 cpath
 
 Same as jpath but parse XML/HTML using CSS selectors from L<Mojo::DOM::CSS>.
@@ -1424,6 +1246,20 @@ Set default I<skipundef> flag for all params in L</vparams> and L</vsort>.
 
 Filters are used in conjunction with types for additional verification.
 
+=head2 min
+
+Check minimum parameter value.
+
+    # Error if myparam less than 10
+    $self->vparam(myparam => 'int', min => 10);
+
+=head2 max
+
+Check maximum parameter value.
+
+    # Error if myparam greater than 100
+    $self->vparam(myparam => 'int', max => 100);
+
 =head2 range
 
 Check parameter value to be in range.
@@ -1452,14 +1288,6 @@ Check maximum length in utf8.
     # Error if value is an empty string
     $self->vparam(myparam => 'str', size => [1, 100]);
 
-=head2 Numbers comparation
-
-I<min>, I<max>, I<equal>, I<not>
-
-=head2 Strings comparation
-
-I<lt>, I<gt>, I<le>, I<ge>, I<cmp>, I<eq>, I<ne>
-
 =head1 RESTRICTIONS
 
 =over
