@@ -1,4 +1,6 @@
 package Mojolicious::Plugin::Vparam::Address;
+use Mojo::Base -strict;
+use Mojolicious::Plugin::Vparam::Common qw(load_class);
 
 use strict;
 use warnings;
@@ -23,6 +25,7 @@ Parse address from string
 sub parse {
     my ($class, $str) = @_;
 
+    return undef unless defined $str;
     my ($full, $address, $lon, $lat, $md5, $id, $type, $lang, $opt);
 
     if( $str =~ m{^\s*\[} and $str =~ m{\]\s*$} ) {
@@ -127,14 +130,49 @@ sub near {
     return $self->opt;
 }
 
+sub check_address($;$) {
+    return 'Value not defined'          unless defined $_[0];
+    return 'Wrong format'               unless ref $_[0];
+    return 'Wrong format'               unless defined $_[0]->address;
+    return 'Wrong format'               unless length  $_[0]->address;
+
+    my $e = load_class('Mojolicious::Plugin::Vparam::Numbers');
+    die $e if $e;
+
+    my $lon = Mojolicious::Plugin::Vparam::Numbers::check_lon( $_[0]->lon );
+    return $lon if $lon;
+
+    my $lat = Mojolicious::Plugin::Vparam::Numbers::check_lat( $_[0]->lat );
+    return $lat if $lat;
+
+    return 'Unknown source'             unless $_[0]->check( $_[1] );
+    return 0;
+}
+
+
+sub register {
+    my ($class, $self, $app, $conf) = @_;
+
+    $app->vtype(
+        address     =>
+            load    => 'Mojolicious::Plugin::Vparam::Address',
+            pre     => sub {
+                return Mojolicious::Plugin::Vparam::Address->parse( $_[1] );
+            },
+            valid   => sub { check_address      $_[1], $conf->{address_secret}},
+    );
+
+    return;
+}
+
 1;
 
-=AUTHORS
+=head1 AUTHORS
 
 Dmitry E. Oboukhov <unera@debian.org>
 Roman V. Nikolaev <rshadow@rambler.ru>
 
-=COPYRIGHT
+=head1 COPYRIGHT
 
 Copyright (C) 2011 Dmitry E. Oboukhov <unera@debian.org>
 Copyright (C) 2011 Roman V. Nikolaev <rshadow@rambler.ru>
